local HenHouse = ...

--------------------------------------------------------------------------
local PV3_HENHOUSE_COMP = {
	TypeName = "PV3_HENHOUSE_COMP",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "FlagTerrain", Type = "integer", Default = 2 ^ OBJECT_FLAG.TERRAIN:toNumber() },
                { Name = "MinHenHouseSpeed", Type = "float", Default = 0.3 },
                { Name = "MaxHenHouseSpeed", Type = "float", Default = 1.5 },
                { Name = "MinHenDeltaX", Type = "float", Default = -1.0 },
                { Name = "MaxHenDeltaX", Type = "float", Default = 1.0 },
                { Name = "MinHenDeltaZ", Type = "float", Default = -1.0 },
                { Name = "MaxHenDeltaZ", Type = "float", Default = 1.0 }
	}
}

--------------------------------------------------------------------------
function PV3_HENHOUSE_COMP:create()
        self._henHouseSpeedCurrent = self.MinHenHouseSpeed
        self._timer = 0
        self._rotationY = 0
        self._henTranslation = { 0, 0, 0 }
        self._henInitialPosition = { 0, 0, 0 }
        self._isHenInitialPositionSet = false
        self.DeltaXThreshold1 = ( self.MaxHenDeltaX - self.MinHenDeltaX ) * ( self.MaxHenDeltaX - self.MinHenDeltaX )
        self.DeltaXThreshold2 = self.DeltaXThreshold1 * 5 / 4
        self.DeltaZThreshold1 = ( self.MaxHenDeltaZ - self.MinHenDeltaZ ) * ( self.MaxHenDeltaZ - self.MinHenDeltaZ )
        self.DeltaZThreshold2 = self.DeltaZThreshold1 * 5 / 4
end

--------------------------------------------------------------------------
function PV3_HENHOUSE_COMP:init()
        if (self:getOwner():getGlobalPosition()[1]*self:getOwner():getGlobalPosition()[1] + self:getOwner():getGlobalPosition()[3]*self:getOwner():getGlobalPosition()[3]) < 2 then
            HenHouse:logWarning("for Maxime: init: self:getOwner():getGlobalPosition()" .. tostring(self:getOwner():getGlobalPosition()))
            return
        end

        self._henInitialPosition = { self:getOwner().Position[1], self:getOwner().Position[2], self:getOwner().Position[3] } 
        self._henInitialGlobalPosition = self:getOwner():getGlobalPosition() 
        self.HenDestination = { self:getOwner().Position[1], self:getOwner().Position[2], self:getOwner().Position[3] }
        self._isHenInitialPositionSet = true
        HenHouse:log("init: self._henInitialPosition" .. tostring(self._henInitialPosition))
        HenHouse:log("init: self:getOwner():getGlobalPosition()" .. tostring(self:getOwner():getGlobalPosition()))
        self:chooseRandomSequence()
        self:getOwner():setRotationY(self._rotationY )
        HenHouse:log("init: FlagTerrain = " .. self.FlagTerrain .. " bit TERRAIN = " .. OBJECT_FLAG.TERRAIN:toNumber())
end


--------------------------------------------------------------------------
function PV3_HENHOUSE_COMP:chooseRandomSequence()
--        HenHouse:log("chooseRandomSequence: parent scale = " .. tostring(self:getOwner():getParent().Scale) .. " rotation = " .. tostring(self:getOwner():getParent().Orientation) .. " position = " .. tostring(self:getOwner():getParent():getGlobalPosition()))
        self._henHouseSpeedCurrent = self.MinHenHouseSpeed + math.random() * (self.MaxHenHouseSpeed - self.MinHenHouseSpeed ) 
        self._henTranslation[1] = self.MinHenDeltaX + math.random() * ( self.MaxHenDeltaX - self.MinHenDeltaX )
        self._henTranslation[2] = self._henInitialPosition[2] 
        self._henTranslation[3] = self.MinHenDeltaZ + math.random() * ( self.MaxHenDeltaZ - self.MinHenDeltaZ )
--        HenHouse:log("chooseRandomSequence: _henInitialPosition = " .. tostring(self._henInitialPosition) .. " _currentPosition = " .. tostring(self:getOwner():getGlobalPosition()).. " Destination = " .. tostring(HenDestination) .. " self._henTranslation = " .. tostring(self._henTranslation))
        local Distance = (self._henTranslation[1]*self._henTranslation[1] + self._henTranslation[3]*self._henTranslation[3]) ^ 0.5
        self._timer = Distance / self._henHouseSpeedCurrent
        if Distance > 0 then
            self._henTranslation[1] = self._henTranslation[1] / Distance
            self._henTranslation[3] = self._henTranslation[3] / Distance
        end
        self:computeHenRotation(Distance)
--        HenHouse:log("chooseRandomSequence: _henInitialPosition = " .. tostring(self._henInitialPosition) .. " self._henTranslation = " .. tostring(self._henTranslation) .. " Distance = " .. Distance)
--        HenHouse:log("chooseRandomSequence: _henInitialPosition = " .. tostring(self._henInitialPosition) .. " _rotationY = " .. self._rotationY .. " self._henHouseSpeedCurrent = " .. self._henHouseSpeedCurrent)
end

--------------------------------------------------------------------------
function PV3_HENHOUSE_COMP:computeHenRotation(_distance)
--        HenHouse:log("computeHenRotation: _henInitialPosition = " .. tostring(self._henInitialPosition) .. " self._henTranslation = " .. tostring(self._henTranslation))
        if _distance > 0 then
            local sinp = self._henTranslation[1] / _distance
            if sinp > 1 then
                self._rotationY = math.pi/2
            elseif sinp < -1 then
               self._rotationY =  -math.pi/2
            else
               self._rotationY = math.asin(sinp)
            end
            if self._henTranslation[3] < 0 then
                self._rotationY = math.pi - self._rotationY
            end
        end 
--        HenHouse:log("computeHenRotation: _henInitialPosition = " .. tostring(self._henInitialPosition) .. " Distance = " .. _distance .. " _rotationY = " .. self._rotationY)
end

--------------------------------------------------------------------------
function PV3_HENHOUSE_COMP:computeGroundElevation(_henPosition)
    local raycastResult2 = {}
--    HenHouse:log("initial raycastResult2 = " .. tostring(raycastResult2))
     
    -- Raycast from the building position + Y = 100
    -- to building position - y = 100
    -- only on objects with a TERRAIN flag
    local FromPosition2 = { _henPosition[1], _henPosition[2]+100, _henPosition[3] }
    local ToPosition2 = { _henPosition[1], _henPosition[2]-100, _henPosition[3] }
--    HenHouse:log("FromPosition2 = " .. tostring(FromPosition2) .. "ToPosition2 = " .. tostring(ToPosition2))
    self:getLevel():rayCast(FromPosition2, ToPosition2, raycastResult2, self.FlagTerrain)
--    HenHouse:log("final raycastResult2 = RigidBody = " .. tostring(raycastResult2["RigidBody"]) .. " Position = " .. tostring(raycastResult2["Position"]) .. " Normal = " .. tostring(raycastResult2["Normal"]))
    if raycastResult2["RigidBody"] == nil then
         HenHouse:logWarning("HenHouse: Hen NOT on ground")
         return 0
    else
--         HenHouse:log("checkGroundElevation = " .. raycastResult2["Position"][2] -_henPosition[2])
         return raycastResult2["Position"][2] -_henPosition[2]
    end
end

--------------------------------------------------------------------------
function PV3_HENHOUSE_COMP:update()
    if not self._isHenInitialPositionSet then
        return
    end

    local dt = self:getLevel():getDeltaTime()
    if dt <= 0 then
        HenHouse:logWarning("HenHouse: dt too small dt = " .. dt)
        return
    end

    if dt > 1 then
        dt = 1
        HenHouse:logWarning("HenHouse: Too much glitch dt = " .. dt)
        self:getOwner():setGlobalPosition(self._henInitialPosition)
    end

    self._timer = self._timer - dt
    if self._timer < 0 then
        self:chooseRandomSequence()
        self:getOwner():setRotationY(self._rotationY)
    end

    -- workaround
    local DeltaX = self:getOwner().Position[1] - self._henInitialPosition[1]
    local DeltaZ = self:getOwner().Position[3] - self._henInitialPosition[3]
        if DeltaX*DeltaX > self.DeltaXThreshold1 then
--            HenHouse:logWarning("HenHouse: Too much glitch DeltaX = " .. DeltaX .. " DeltaZ = " .. DeltaZ)
            self._henTranslation[1] = -self._henTranslation[1]
            self:computeHenRotation((self._henTranslation[1]*self._henTranslation[1] + self._henTranslation[3]*self._henTranslation[3]) ^ 0.5)
            self:getOwner():setRotationY(self._rotationY )
        end
        if DeltaZ*DeltaZ > self.DeltaZThreshold1 then
--            HenHouse:logWarning("HenHouse: Too much glitch DeltaX = " .. DeltaX .. " DeltaZ = " .. DeltaZ)
            self._henTranslation[3] = -self._henTranslation[3]
            self:computeHenRotation((self._henTranslation[1]*self._henTranslation[1] + self._henTranslation[3]*self._henTranslation[3]) ^ 0.5)
            self:getOwner():setRotationY(self._rotationY )
        end
    local DeltaGlobalX = self:getOwner():getGlobalPosition()[1] - self._henInitialGlobalPosition[1]
    local DeltaGlobalZ = self:getOwner():getGlobalPosition()[3] - self._henInitialGlobalPosition[3]
        if DeltaGlobalX*DeltaGlobalX > self.DeltaXThreshold2 then
--            HenHouse:logWarning("HenHouse: Too much glitch DeltaGlobalX = " .. DeltaGlobalX .. " DeltaGlobalZ = " .. DeltaGlobalZ)
            self:getOwner():setGlobalPosition(self._henInitialGlobalPosition)
            return
        end
        if DeltaGlobalZ*DeltaGlobalZ > self.DeltaZThreshold2 then
--            HenHouse:logWarning("HenHouse: Too much glitch DeltaGlobalX = " .. DeltaGlobalX .. " DeltaGlobalZ = " .. DeltaGlobalZ)
            self:getOwner():setGlobalPosition(self._henInitialGlobalPosition)
            return
        end

    local HenTranslation = { 
        self._henTranslation[1]*dt*self._henHouseSpeedCurrent,
        self:computeGroundElevation(self:getOwner():getGlobalPosition()),
        self._henTranslation[3]*dt*self._henHouseSpeedCurrent
    }

--  HenHouse:log("Update: dt = " .. dt .. " Distance = " .. Distance)
    self:getOwner():translate(HenTranslation)        
end

--------------------------------------------------------------------------
HenHouse:registerClass(PV3_HENHOUSE_COMP)

--------------------------------------------------------------------------
local PV3_HENHOUSE_DOOR_COMP = {
	TypeName = "PV3_HENHOUSE_DOOR_COMP",
	ParentType = "COMPONENT",
	Properties = {
                { Name = "HenHouseDoorRotation", Type = "float", Default = 4 * math.pi / 10 },
                { Name = "HenHouseDoorRotationalSpeed", Type = "float", Default = 0.5 },
	}
}

--------------------------------------------------------------------------
function PV3_HENHOUSE_DOOR_COMP:create()
end

--------------------------------------------------------------------------
function PV3_HENHOUSE_DOOR_COMP:init()
    self._sequence = 0
    self._initialOrientation = self:getOwner():getGlobalOrientation()
end

--------------------------------------------------------------------------
function PV3_HENHOUSE_DOOR_COMP:onFinalize()
--    self:henHouseDoorRotation(-1)
end

--------------------------------------------------------------------------
function PV3_HENHOUSE_DOOR_COMP:onEnabled()
--    HenHouse:log("onEnabled: _deltaRotationY = " .. tostring(self._deltaRotationY))
    if self._sequence == 0 then
            self:henHouseDoorRotation(1.0)
            self._sequence = 1 -- request on going
    end
end

--------------------------------------------------------------------------
function PV3_HENHOUSE_DOOR_COMP:onDisabled()
 --   HenHouse:log("onDisabled: _deltaRotationY = " .. tostring(self._deltaRotationY))
    if self._sequence == 0 then
        if not (self._deltaRotationY < 0) then
            self:henHouseDoorRotation(-1.0)
            self._sequence = 1  -- request on going
        end
    end
end

--------------------------------------------------------------------------
function PV3_HENHOUSE_DOOR_COMP:henHouseDoorRotation(_senseOfRotation)
        self._timer = self.HenHouseDoorRotation / self.HenHouseDoorRotationalSpeed
        self._deltaRotationY = _senseOfRotation

--        HenHouse:log("henHouseDoorRotation: _deltaRotationY = " .. tostring(self._deltaRotationY))
end

--------------------------------------------------------------------------
function PV3_HENHOUSE_DOOR_COMP:update()
    local dt = self:getLevel():getDeltaTime()
    if self._timer < 0 then
        if self._deltaRotationY < 0 then
            self:getOwner():setGlobalOrientation(self._initialOrientation)
        end
        self._sequence = 0  -- no request on going
        return
    end
    self._timer = self._timer - dt
--    HenHouse:log("update: _deltaRotationY = " .. tostring(self._deltaRotationY))

    self:getOwner():rotateY(self._deltaRotationY * dt * self.HenHouseDoorRotationalSpeed)
end

--------------------------------------------------------------------------
HenHouse:registerClass(PV3_HENHOUSE_DOOR_COMP)
